@@ -4,7 +4,10 @@ import asyncio
 import functools
 import itertools
 import random
-from numbers import Number
+from collections.abc import Iterator
+from pathlib import Path
+
+import mido
 
 from musictool import config
 from musictool.note import Note
@@ -35,6 +38,8 @@ class Chord(NoteSet):
 
     }
     name_to_intervals = {v: k for k, v in intervals_to_name.items()}
+    root: Note
+    name: str
 
 
 class SpecificChord(Cached):
@@ -64,11 +69,9 @@ class SpecificChord(Cached):
         self.key = self.notes, self.root
 
     @classmethod
-    def random(cls, n_notes=None, octaves=None) -> SpecificChord:
+    def random(cls, n_notes: int | None = None, octaves: tuple[int, ...] = (3, 4, 5)) -> SpecificChord:
         if n_notes is None:
             n_notes = random.randint(2, 5)
-        if octaves is None:
-            octaves = 3, 4, 5
         notes_space = tuple(
             SpecificNote(note, octave)
             for note, octave in itertools.product(config.chromatic_notes, octaves)
@@ -78,25 +81,23 @@ class SpecificChord(Cached):
 
     @classmethod
     def from_str(cls, string: str) -> SpecificChord:
-        notes, _, root = string.partition('/')
-        root = Note(root) if root else None
-        notes_ = notes.split('_')
-        if len(notes_) != len(set(notes_)):
+        notes_, _, root_ = string.partition('/')
+        root = Note(root_) if root_ else None
+        notes_2 = notes_.split('_')
+        if len(notes_2) != len(set(notes_2)):
             raise NotImplementedError('SpecificChord_s with non unique notes are not supported')
-        notes = frozenset(SpecificNote.from_str(note) for note in notes_)
+        notes = frozenset(SpecificNote.from_str(note) for note in notes_2)
         return cls(notes, root=root)
 
-    def notes_combinations(self, ids=False):
-        if ids: yield from itertools.combinations(range(len(self.notes_ascending)), 2)
-        else: yield from itertools.combinations(self.notes_ascending, 2)
-        # for n, m in itertools.combinations(self.notes_ascending, 2):
-        #     yield n, m
+    def notes_combinations(self,) -> Iterator[tuple[SpecificNote, SpecificNote]]:
+        yield from itertools.combinations(self.notes_ascending, 2)
 
-    def find_intervals(self, interval: int):
+    def find_intervals(self, interval: int) -> tuple[tuple[SpecificNote, SpecificNote], ...]:
         return tuple((n, m) for n, m in self.notes_combinations() if abs(m - n) == interval)
 
     def __len__(self): return len(self.notes)
-    def __getitem__(self, item): return self.notes_ascending[item]
+    def __getitem__(self, item: int) -> SpecificNote: return self.notes_ascending[item]
+    def __iter__(self): return iter(self.notes_ascending)
 
     def __repr__(self):
         _ = '_'.join(repr(note) for note in self.notes_ascending)
@@ -114,7 +115,7 @@ class SpecificChord(Cached):
     #     """
     #     return sum(note.absolute_i for note in self.notes) - sum(note.absolute_i for note in other.notes)
 
-    def __sub__(left, right):
+    def __sub__(left, right: SpecificChord) -> int:
         return sum(abs(l - r) for l, r in zip(left, right))
 
     def __add__(self, other: int) -> SpecificChord:
@@ -128,7 +129,7 @@ class SpecificChord(Cached):
     def transposed_to_C0(self) -> SpecificChord:
         return self + (SpecificNote('C', 0) - self[0])
 
-    async def play(self, seconds: Number = 1, bass_octave: int | None = None) -> None:
+    async def play(self, seconds: float = 1, bass_octave: int | None = None) -> None:
         tasks = [note.play(seconds) for note in self.notes]
         if bass_octave:
             if self.root is None:
@@ -136,8 +137,11 @@ class SpecificChord(Cached):
             tasks.append(SpecificNote(self.root, bass_octave).play(seconds))
         await asyncio.gather(*tasks)
 
-    def to_midi(self, path=None, n_bars=1):
-        import mido
+    def to_midi(
+        self,
+        path: str | Path | None = None,
+        n_bars: int = 1,
+    ) -> mido.MidiFile | None:
         mid = mido.MidiFile(type=0, ticks_per_beat=96)
         track = mido.MidiTrack()
         track.append(mido.MetaMessage(type='track_name', name='test_name'))
@@ -158,3 +162,4 @@ class SpecificChord(Cached):
         if path is None:
             return mid
         mid.save(path)
+        return None
