@@ -2,15 +2,19 @@ from __future__ import annotations
 
 import itertools
 import random
+from typing import Type
+from typing import TypeVar
+from typing import no_type_check
+from typing import overload
 
 import pipe21 as P
 
-from musictool import chromatic
 from musictool import config
 from musictool.note import AnyNote
 from musictool.note import Note
 from musictool.note import SpecificNote
 from musictool.note import str_to_note
+from musictool.util import typeguards
 from musictool.util.cache import Cached
 
 '''
@@ -21,7 +25,8 @@ NoteSet
 '''
 
 
-def bits_to_intervals(bits: str) -> frozenset:
+@no_type_check
+def bits_to_intervals(bits: str) -> frozenset[int]:
     return (
         bits
         | P.Map(int)
@@ -33,7 +38,7 @@ def bits_to_intervals(bits: str) -> frozenset:
     )
 
 
-def intervals_to_bits(intervals: frozenset) -> str:
+def intervals_to_bits(intervals: frozenset[int]) -> str:
     bits = ['0'] * 12
     bits[0] = '1'
     for i in intervals:
@@ -41,9 +46,15 @@ def intervals_to_bits(intervals: frozenset) -> str:
     return ''.join(bits)
 
 
+Self = TypeVar('Self', bound='NoteSet')
+
+
 class NoteSet(Cached):
-    intervals_to_name: dict = {}
-    name_to_intervals: dict = {}
+    intervals_to_name: dict[frozenset[int], str] = {}
+    name_to_intervals: dict[str, frozenset[int]] = {}
+
+    notes: frozenset[Note]
+    root: Note | None
 
     def __init__(
         self,
@@ -63,32 +74,41 @@ class NoteSet(Cached):
         if len(notes) == 0:
             raise ValueError('notes should be not empty')
 
-        if isinstance(next(iter(notes)), str):
-            notes = frozenset(Note(note) for note in notes)
+        if typeguards.is_frozenset_of_str(notes):
+            self.notes = frozenset(Note(note) for note in notes)
+        elif typeguards.is_frozenset_of_note(notes):
+            self.notes = notes
+        else:
+            raise TypeError
 
-        if isinstance(root, str):
-            root = Note(root)
+        self.notes_octave_fit = tuple(sorted(self.notes))
 
-        if root is not None and root not in notes:
-            raise KeyError('root should be one of notes')
+        if root is None:
+            self.root = root
+            self.notes_ascending = self.notes_octave_fit
+        else:
+            if root not in self.notes:
+                raise KeyError('root should be one of notes')
 
-        self.notes = notes
-        self.root = root
-        self.key = self.notes, self.root
-        self.notes_ascending = chromatic.sort_notes(self.notes, start=self.root)
-        self.notes_octave_fit = tuple(sorted(notes))
-        self.note_i = {note: i for i, note in enumerate(self.notes_ascending)}
-        self.increments = {a: b - a for a, b in itertools.pairwise(self.notes_ascending + (self.notes_ascending[0],))}
-        self.decrements = {b: -(b - a) for a, b in itertools.pairwise((self.notes_ascending[-1],) + self.notes_ascending)}
-        if root is not None:
-            if root not in notes:
-                raise ValueError('root note should be one of the chord notes')
+            if isinstance(root, Note):
+                self.root = root
+            elif isinstance(root, str):
+                self.root = Note(root)
+            else:
+                raise TypeError('root type should be str | Note | None')
 
-            self.intervals_ascending = tuple(note - root for note in self.notes_ascending)
+            root_i = self.notes_octave_fit.index(self.root)
+            self.notes_ascending  = self.notes_octave_fit[root_i:] + self.notes_octave_fit[:root_i]
+            self.intervals_ascending = tuple(note - self.root for note in self.notes_ascending)
             self.intervals = frozenset(self.intervals_ascending[1:])
             self.bits = intervals_to_bits(self.intervals)
             self.name = self.__class__.intervals_to_name.get(self.intervals)
 
+        self.key = self.notes, self.root
+        self.note_i = {note: i for i, note in enumerate(self.notes_ascending)}
+        self.increments = {a: b - a for a, b in itertools.pairwise(self.notes_ascending + (self.notes_ascending[0],))}
+        self.decrements = {b: -(b - a) for a, b in itertools.pairwise((self.notes_ascending[-1],) + self.notes_ascending)}
+
     @property
     def rootless(self): return NoteSet(self.notes)
 
@@ -98,31 +118,37 @@ class NoteSet(Cached):
         return NoteSet.from_intervals(self.intervals, note)
 
     @classmethod
-    def from_name(cls, root: str | Note, name: str) -> NoteSet:
+    def from_name(cls: Type[Self], root: str | Note, name: str) -> Self:
         if isinstance(root, str):
             root = Note(root)
         notes = frozenset(root + interval for interval in cls.name_to_intervals[name]) | {root}
         return cls(notes, root=root)
 
     @classmethod
-    def from_intervals(cls, intervals: frozenset, root: str | Note):
+    def from_intervals(cls: Type[Self], intervals: frozenset[int], root: str | Note) -> Self:
         if isinstance(root, str):
             root = Note(root)
         return cls(frozenset(root + interval for interval in intervals) | {root}, root=root)
 
     @classmethod
-    def random(cls, n_notes=None):
+    def random(cls: Type[Self], n_notes: int | None = None) -> Self:
         if n_notes is None:
             n_notes = random.randint(2, 5)
         notes = frozenset(random.sample(config.chromatic_notes, n_notes))
         return cls(notes)
 
     @classmethod
-    def from_str(cls, string: str):
-        notes, _, root = string.partition('/')
-        root = Note(root) if root else None
-        notes = frozenset(Note(note) for note in notes)
-        return cls(notes, root=root)
+    def from_str(cls: Type[Self], string: str) -> Self:
+        notes, _, root_ = string.partition('/')
+        root = Note(root_) if root_ else None
+        return cls(frozenset(Note(note) for note in notes), root=root)
+
+    @overload
+    def add_note(self, note: str, steps: int) -> Note | SpecificNote: ...
+    @overload
+    def add_note(self, note: SpecificNote, steps: int) -> SpecificNote: ...
+    @overload
+    def add_note(self, note: Note, steps: int) -> Note: ...
 
     def add_note(self, note: AnyNote, steps: int) -> Note | SpecificNote:
         notes = self.notes_ascending
@@ -151,7 +177,8 @@ class NoteSet(Cached):
     def __hash__(self): return hash(self.key)
     def __len__(self): return len(self.notes)
     def __getitem__(self, item): return self.notes_ascending[item]
-    def __contains__(self, item): return item in self.notes
+    def __iter__(self): return iter(self.notes_ascending)
+    def __contains__(self, item: str | Note) -> bool: return item in self.notes
     def __le__(self, other): return self.notes <= other
     def __ge__(self, other): return other <= self.notes
 
@@ -229,7 +256,9 @@ class NoteRange:
         self._key = self.start, self.stop, self.noteset
 
     def _getitem_int(self, item: int) -> SpecificNote:
-        if 0 <= item < len(self): return self.noteset.add_note(self.start, item)
+        if 0 <= item < len(self):
+            q = self.noteset.add_note(self.start, item)
+            return q
         elif -len(self) <= item < 0: return self.noteset.add_note(self.stop, item + 1)
         else: raise IndexError('index out of range')
 
@@ -239,9 +268,10 @@ class NoteRange:
             if not 0 <= item.start <= item.stop <= len(self):
                 raise IndexError('NoteRange slice is out of range')
             return NoteRange(self._getitem_int(item.start), self._getitem_int(item.stop), self.noteset)
-        else: raise TypeError('NoteRange indices must be integers or slices, not str')
+        else: raise TypeError(f'NoteRange indices must be integers or slices, got {type(item)}')
 
-    def __contains__(self, item: SpecificNote): return item.abstract in self.noteset and self.start <= item <= self.stop
+    def __iter__(self): return (self[i] for i in range(len(self)))
+    def __contains__(self, item: SpecificNote) -> bool: return item.abstract in self.noteset and self.start <= item <= self.stop
     def __repr__(self): return f'NoteRange({self.start}, {self.stop}, noteset={self.noteset})'
     def __len__(self): return self.noteset.subtract(self.stop, self.start) + 1
     def __eq__(self, other): return self._key == other._key
