@@ -1,4 +1,3 @@
-import collections
 
 from musictool.chord import SpecificChord
 from musictool.note import SpecificNote
@@ -11,7 +10,7 @@ class Transition:
         self.b = b
 
     @staticmethod
-    def arrow(a: SpecificNote, b: SpecificNote):
+    def arrow(a: SpecificNote, b: SpecificNote) -> str:
         if a < b: return '︎\\'
         elif a == b: return '|'
         else: return '/'
@@ -44,16 +43,16 @@ def chord_transitions(
 
 
 def transition_graph(start_chord: SpecificChord, noterange: NoteRange) -> dict[SpecificChord, frozenset[SpecificChord]]:
-    graph = collections.defaultdict(set)
+    # graph: dict[SpecificChord, set[SpecificChord]] = collections.defaultdict(set)
+    graph = dict()
 
-    def _graph(chord: SpecificChord):
+    def _graph(chord: SpecificChord) -> None:
         if chord in graph:
             return
         childs = chord_transitions(chord, noterange)
-        graph[chord] |= childs
+        graph[chord] = childs
         for child in childs:
             _graph(child)
 
     _graph(start_chord)
-    graph = dict(graph)
     return graph
